//  
//  
clear;  // Variablen lschen, 
// Command-Fenster lschen mit e2
printf('\n\n_______________________________________________________________________\n\n');
printf('\n\nApproximation der Planck-Temperatur \naus dem Pegel zweier Spektrallinien\n\n');
printf('Autor: G. Heinz, GFaI-Berlin, 12/2007 - 1/2008\n');
printf('heinz@gfai.de, www.gfai.de/~heinz\n');
printf('Copyrights: Uneingeschrnkte Nutzung erlaubt bei Quellenverweis.\n\n');
author=' www.gfai.de/~heinz';    // Diese Zeile darf nicht entfernt werden.
printf('Input:  Emissionspegel e1 and e2 bei Wellenlnge L1 and L2 in m\n');
printf('Output: Temperaturfunktion T = f(e1/e2)\n\n');
printf('Absolutpegel von e1 und e2 sind unbekannt -> Kalibrierung mit tempk\n');

//printf('For Scilab-3.1.1 see http://www.scilab.org/\n\n');

// Programm mit Initialisierungsparameterfile *.pin

// Ausfhrung von der Befehlszeile:
// exec(getcwd()+'\tempfunktion_entwickeln.sce',-1)

// Grundidee:
// Sind die Emissionskoeffizienten e1, e2 zweier Linien der Wellenlngen L1, L2 bekannt, 
// so lt sich daraus unter Annahme 
// rein Planckscher Strahlung die Temperatur schtzen. 
// Wir bentigen explizit eine (Quotienten-) Funktion beider Linien. 
// Diese lt sich ber die Plancksche Strahlungsgleichung erzeugen. 
// Sie wird als Matrix ausgegeben.

// Planck-Kurve =  f(lambda_max, T)    mit  T: Temperatur und lambda_max: Temperaturmaximum
// Numerische Inversion zur Zielfunktion: T = f( L1, L2)
//
// Es werden Planckkurven fr alle Temperaturen im Sttzstellenabstand dT entwickelt.
// Temperatur wird fr die eingesetzten Lambda in Form vierer=[grad,e1,e2,e1zue2] protokolliert
//
// Test: Programm ausfhren, dann in Controlpanel eingeben: vierer
//   vierer=[grad,e1,e2,e1zue2] bzw.  
//   planck=[grad,e1,e2] bzw.
//   zweier=[e1zue2,grad]
// 
// Als File wird Temperaturfunktion gespeichert im Matrix-Format: 
// vierer=[grad,e1,e2,e1zue2]   // Achtung: Variablenname 'vierer' wird mit gespeichert als 
// z.B. tempfkt_440nm_740nm_199x4.bin
// dabei kennzeichnet 199x4 die Matrixdimension: 199 Werte in 4 Spalten
//
// Matrix vierer aus dem File wiederholen?
// clear vierer; // lscht die Variable vierer
// load(wrfile); // Fileinhalt ist dann unter dem Variablennamen vierer wieder abrufbar (Filename ist wrfile)
//
// Zugriff auf die Spalten:
//        grad=vierer(:,1);   // erste Spalte: Temperatur
//        e1=vierer(:,2);     // zweite Spalte: e1-Linie
//        e2=vierer(:,3);     // dritte Spalte: e2-Linie
//        e1zue2=vierer(:,4); // vierte Spalte: Verhltnis e1/e2
// 
// Die Plots zeigen:
// Scilab Graphic (0): die Planckkurvenschar (man setze dT=1000 fr bessere Sichtbarkeit) 
// Scilab Graphic (1): die Temperatur in Abhngigkeit von den Werten fr e1 und e2
// Scilab Graphic (2): Multiplikation und Division der Funktionen e1 und e2
// Scilab Graphic (3): die Temperaturfunktion T = f(e1/e2)
// 

      
//  Default-Parameter-Init: 
      // wird durch Wert im Parameterfile *.pin berschrieben
      
      // Bestimmungs-Wellenlngen:
      L1=0.44;   // Empfindlichkeitsmaximum erster Photodiode in m 
      L2=0.74;   // Empfindlichkeitsmaximum zweiter Photodiode in m 
                 // Beispiel: AG38: 380nm, BP104: 950nm
      Ln=300;         // Anzahl der lambda-Sttzpunkte pro Kurve
      Lstart=.2;      // Von Wellenlnge [in m]
      Lstop=.79;    // Bis Wellenlnge [in m]
      m=0.000001;     // Korrekturfaktor, wenn lambda in Mikrometer eingesetzt wird
      col='black';    // Farbe des Ergebnisplots #3      
      
      // Temperaturen:
      Tanf=250;    // Beginnen bei Temperatur
      Tend=15250;  // Endtemperatur
      T=Tanf;      // Temperatur in Kelvin
      dT= (Tend-Tanf)/Ln ;     // Temperatur-Schrittweite fr Ablage in *.bin-File 
      tempk = 1;   // Korrektur der Temperatur - Anpassung der Linien an Realtemperatur
      Mratio=100;    // plottbare Dekaden in Fenster #0 bei log. Darstellung
      
      // Konstanten:
          // Float-Formate: D, e, f
          // global h; global k; global c;
          // normiertes System: V,A,s,Hz,m
      h = 6.626D-34;    // Plancksches We2kungsquantum in [W*s*s]
      c = 3D8;          // Lichtgeschwindigkeit in [m/s]
      k = 1.38D-23;     // Boltzmann-Konstante in [W*s/K]
      d = 5.67D-8;      // Stefan-Boltzmann-Konstante in [W/(m^2 K^4)]

      // hck = h*c/k; // [W*s*s] * [m/s] / [W*s/K] = [K*m] (Kelvin mal Meter)
      // hk = h/k     // [W*s*s] / [W*s/K] = [K*s] (Kelvin mal Sekunden);


// Parameterfile *.pin holen
      path=xgetfile('*.pin',title='Open parameter file (*.pin)');
      exec(path);  // ldt Variablennamen mit File-Inhalt
      printf("\nFrom parameter file: \n%s\n\n",path);
      //who_user;  // Anzeige der Variablen


// Strings basteln:
      values = ';   L1 = '+string(L1*1e3)+'nm, '+'L2 = '+string(L2*1e3)+'nm ';      // fr GIFs
      filename = 'tempfkt_'+string(L1*1e3)+'nm'+'_'+string(L2*1e3)+'nm';      // fr Ausgabefiles


// Los gehts!
      // printf('\n%s',titel);
      // printf('\nfilename = %s, \nL1 = %f nm, \nL2 = %f nm \n\n',filename, L1, L2);
            
      printf('L1 = %f m \nL2 = %f m \ndT = %f K \n\n',L1,L2,dT);
      
//  
//  Fe Eisen:
//  Schmelzpunkt 	1808 K (1535 C)
//  Siedepunkt 	3023 K (2750 C)
//  
//  Cu Kupfer
//  Schmelzpunkt 	1357,6 K (1084,4 C)
//  Siedepunkt 	2840 K (2567 C)
//  
//  Mn Mangan
//  Schmelzpunkt 	1517 K (1244 C)
//  Siedepunkt 	2235 K (1962 C)
//  
//  Mg Magnesium
//  Schmelzpunkt 	923 K (650 C)
//  Siedepunkt 	1380 K (1107 C)
//  
//  Zn Zink
//  Schmelzpunkt 	692,68 K (419,5 C)
//  Siedepunkt 	1180 K (907 C)
//  
//  
//  
//  Zur Erinnerung:
//  
//  h=6.626e-34 Js    Plancksches We2kungsquantum
//  h=6.626e-34 Ws^2  
//  1 J = 1 Ws        Joule
//  1 W = 1 VA        Leistung
//  pi=3.141          Pi
//  c=3e8 m/s         Lichtgeschwindigkeit
//  k=1.38e-23 J/K    Boltzmann-Konstante
//  T=5000 K          Temperatur
//  lambda = 0.5 m = 5e-7 m = 1/f    Wellenlnge in m (!) Normierung mit m
//  L = W / (m^2 Hz sr) = E*lambda / A    spektrale Strahlungsdichte
//  
//  Frequenzform der Planckschen Strahlungsglg.:
//  L(f,T) =(2*pi/c^2) * (h*f^3/(exp(h*f/(k*T))-1))    [W/(mHz)];
//  
//  Wellenlngenform (hierfr):
//  L(lambda, T) = 2*h*c^2/lambda^5 * 1/(exp(h*c/(lambda*k*T))-1)  [W/m m]
//  
//  


// Bildaufbau abwarten...

      winId=waitbar('... Bitte warten ...');
      printf("... Bitte warten ...\n\n");
      realtimeinit(0.3);
      for j=0:0.1:1,
          realtime(3*j);
          waitbar(j,winId);
      end
      winclose(winId);



// Menue basteln?
     // addmenu(Parameter)
     // function H=Parameter(T)...
     // delmenu(Parameter)



// ============= Planck-Funktion =============

       // Emission als Funktion der Wellenlnge und Temperatur
       function y=planck(lambda,T);   // y = spek(h,c,lambda,k,T)   
            // Spektrale Strahlungsdichte in [Watt/(m m)]
            // skalare (elementweise) Operationen mit Dot (.) davor !!!
            // Skalierung der Kurven auf gleiche Hhe mit 1/T^5 (normiert):
            y = m .*2 .*h .*c.^2 ./((m.*lambda).^5) .*1 ./(exp(h.*c./(k.*m.*lambda.*T))-1);   //  y pro m, dazu Korrektur mit m
            // y = 1 ./T.^5 .*m .*2 .*h .*c.^2 ./(m.*lambda.^5) .*1 ./(exp(h.*c./(k.*m.*lambda.*T))-1);   //  normierte Darstellung
       endfunction;    // skalare (elementweise) Operationen mit Dot (.) davor !!!
       
       if type(whereis(planck))<>1, then comp(planck); end;   // Wenn noch nicht getan, compiliere die Funktion planck (schneller)



// ============= Programm ====================


// Nulltes Plotbild init

      scf(0);   // Fenster #0
      clf(0);   // Fenster lschen

 
// Schleife ber alle T (Temperaturen) und alle n (Wellenlngen)

      lambda=linspace(Lstart,Lstop,Ln); // linspace generiert Ln Elemente per default, lambda-Matrix in m
      n=1;  // n mitzhlen fr Test

      while T < Tend;
                y=planck(lambda,T);      // Berechnung: lambda ist Vektor der zu berechnenden Linien
                // printf('Spektrale Strahlungsdichte y=%e Watt/(m m) bei T=%i K\n\n', y(1),T(1));
                // Maximum suchen:
                  [yval,ypos]=sort(y);  // zuordnen auf yval: Vektor der Werte und ypos: Positionsnummernvektor
                  ymax=yval(1);         // max. Empfindlichkeit steht auf Position 1, dh. ymax=max(yval)
                  // die Wellenlnge des ersten Elements des Positionsvektors ypos ist xmax:
                  xmax=lambda(ypos(1)); // xmax: Wellenlnge bei maximaler Amplitude (fr Plotlabel)
                
                colorid=color(col);     // col ist String "red", colorid ist number: 5.
                drawlater();            // jetzt noch nicht zeichnen, dauert zu lange
                // Nulltes Fenster plotten
                plot2d(lambda,y,style=colorid)  // lambda und y sind Vektoren der Lnge n
                handl=get("current_axes"); // get the handle of the newly created axes
                handl.log_flags="nl";      // n: nichtlog, l: log, Reihenfolge xy
                handl.tight_limits="on";   // wird fr nchsten Befehl gebraucht bei log.
                handl.data_bounds=[Lstart,ymax/Mratio;Lstop,ymax]; // Anzeigebereich fixieren bei log.
                // plot2d(lambda,planck(lambda,T),style=colorid)  // nicht normierte Darstellung
                // plot2d1("oln",lambda,y)  // anderes logarithmisches Plot
                Tstring=string(T);      // Temperatur fr Label zum String konvertieren
                xstring(xmax,ymax*.8,Tstring,90,0) // Labelkoord.: (xmax, ymax, name, rotation, box)
                // plot(lambda,y);      // Testplot
                // legends(Tstring,1,,opt="lr");
                // who_user();
           
                // jetzt die Spalten fr L1 und L2 berechnen:
                ye1(n)=planck(L1,T);
                ye2(n)=planck(L2,T);
                grad(n)=T;
                
                n=n+1;    // next n
                T=T+dT;   // 
      end; // end while


// Nulltes Fenster plotten und GIF schreiben:

      xselect();  // Fenster auf!
      drawnow();    // erst jetzt Plot ausfhren um Zeit zu sparen
      titel='Planck-Emission M = planck(T,lambda)    '+'Parameter: Temperature T in Kelvin@'+author;
      yunit='M in Watt/(m m)';  // yunit="Watt/(K^5 m m)";  // fr normierte Darstellung
      xunit='lambda in m';
      xtitle(titel,xunit,yunit);    // Titel und Maeinheiten ranschreiben
      filena=filename+"_0_planck.gif";   // Filenamen zusammenbasteln
      xs2gif(0,filena); // GIF von Fenster #0 machen
      printf('... GIF-Plot of Window %i saved as %s \n\n',0,filena);

// Planck normieren:
      
    // Kurvenschar normieren  e2:
      // ye2=ye2./T^5;    // Temperatur eliminieren
      [ye2v,ye2p]=sort(ye2);  // Sortieren zweispaltig, Value and Position (v und p)
      ye2x=ye2v(1);   // Maximumwert ermitteln
      e2=ye2/ye2x;    // damit normieren auf 1.
      e2 = e2 / tempk;      // Kalibrierung der Temperatur
      
    // Kurvenschar normieren  e1:
      // ye1=ye1./T^5;    // Temperatur eliminieren      
      [ye1v,ye1p]=sort(ye1);  // Sortieren zweispaltig, Wert und Position merken
      ye1x=ye1v(1);   // Maximumwert ermitteln
      e1 = ye1/ye1x;    // Normieren auf 1.
      e1 = e1 * tempk;      // Kalibrierung der Temperatur
      
      
    // e1 zu e2 berechnen:
      e1zue2 = e1 ./ e2;   // Verhltnis e1 zu e2 berechnen


    // Planck:         Dreier-Matrix bilden
      planck=[grad,e1,e2];  // 3-spaltige Matrix der Lnge n erzeugen planck(grad,ye1,ye2)
          // Zugriff darauf zum Beispiel so:
          // grad=planck(:,1);   // erste Spalte
          // e1=planck(:,2);     // zweite Spalte
          // e2=planck(:,3);     // dritte Spalte
            
    // Vierer:         Vierer-Matrix bilden
      vierer=[grad,e1,e2,e1zue2];
      
    // Zweier:         Zweier-Matrix bilden
      zweier=[e1zue2,grad];     // Funktion reicht fr Zuordnung zur Temperatur, da monotoner Anstieg!

      
// Write Temperaturfile

      anz=size(vierer); // ergibt z.B. --> ! 149. 4. !   d.h. Matrix (149 x 4)
      reih=anz(1,1);    // erste Reihe, erste Spalte (149)
      spalt=anz(1,2);   // erste Reihe, zweite Spalte (4)
      
      wrfile=filename+'_'+string(reih)+'x'+string(spalt)+'.bin';   // Matrix mit Reihen und Spalten
      save(wrfile,vierer); // 
      
      printf('Achtung: Variablenname vierer wird mit gespeichert\n\n');
      printf('Temperature function saved as  \nwrfile = %s\n',wrfile);
      printf('with matrix-format vierer=[grad,e1,e2,e1zue2]\n\n');

      // Test:
      // clear vierer
      // File wieder lesen?
      // load(wrfile); // Fileinhalt ist nun unter dem Variablennamen abrufbar, hier: vierer
   
      
// Erstes Fenster Plot Ortskurve
      // printf('Attention: Matrix-multiplication e1*e2 for Graphic (1)\n -> crash, if dT is too small\n\n');
      scf(1);   // Fenster 1
      clf(1);   // lschen
      xselect();    // Fenster auf!
      // plot2d(ye2,ye1,style=colorid);
      e1mat=[grad,e1];
      e2mat=[grad,e2];
      gradmat=e1mat(:,1)*e2mat(:,1)';   // Spalte mal Zeile = Quadratmatrix bilden
      xset("colormap",jetcolormap(64));
      colorbar(0,Tend,[1,64]);
      grayplot(e2,e1,gradmat); 
      xtitle("T = f(e1,e2)"+values+" @"+author+", T in Kelvin","e2@"+string(L2)+" m","e1@"+string(L1)+" m");
      filena=filename+"_1.gif";
      xs2gif(1,filena); // GIF von Fenster #1 machen
      printf('... GIF-Plot of Window %i saved as %s \n\n',1,filena);
     
      
// Zweites Fenster  Plot (Blau, Rot, Blau/Rot, Blau-Rot)

      scf(2);   // Fenster 2
      clf(2);   // lschen
      
      rmb=e2-e1;         // Plot2
      bmr=e1-e2;         // Plot2
      mul=e1.*e2;        // Plot2

      xselect();    // Fenster auf!
      plot2d(grad/1000,[bmr,e1zue2,e1,e2],style=[color('green'),color('black'),color('blue'),color('red')])
      // legends(['e1','e2','e1-e2','e1/e2'],[[color('blue');1],[color('red');1],[color('green');1],[color('black');1]],with_box=%f,opt="lr");
      // legends(['Blue '+string(L1)+' m','Red '+string(L2)+' m','Blue-Red','Blue/Red'],[color('blue'),color('red'),color('green'),color('black')],with_box=%f,opt="ul"); // upper/lower; left/right
      legends(['e1 for L1 = '+string(L1)+' m','e2 for L2 = '+string(L2)+' m','e1-e2','e1/e2'],[color('blue'),color('red'),color('green'),color('black')],with_box=%f,opt="ul"); // upper/lower; left/right
      filena=filename+"_2_b-r.gif";
      xs2gif(2,filena); // GIF von Fenster #2 machen
      
      
      scf(2);   // Fenster 2
      clf(2);   // lschen
      xselect();    // Fenster auf!
      plot2d(grad/1000,[mul,e1zue2,e1,e2],style=[color('green'),color('black'),color('blue'),color('red')])
      // legends(['e1','e2','e1-e2','e1/e2'],[[color('blue');1],[color('red');1],[color('green');1],[color('black');1]],with_box=%f,opt="lr");
      // legends(['Blue '+string(L1)+' m','Red '+string(L2)+' m','Blue*Red','Blue/Red'],[color('blue'),color('red'),color('green'),color('black')],with_box=%f,opt="ul"); // upper/lower; left/right
      legends(['e1 for L1 = '+string(L1)+' m','e2 for L2 = '+string(L2)+' m','e1*e2','e1/e2'],[color('blue'),color('red'),color('green'),color('black')],with_box=%f,opt="ul"); // upper/lower; left/right
      filena=filename+"_2_mult.gif";
      xs2gif(2,filena); // GIF von Fenster #2 machen

      
      scf(2);   // Fenster 2
      clf(2);   // lschen
      xselect();    // Fenster auf!
      plot2d(grad/1000,[rmb,e1zue2,e1,e2],style=[color('green'),color('black'),color('blue'),color('red')])
      // legends(['e1','e2','e1-e2','e1/e2'],[[color('blue');1],[color('red');1],[color('green');1],[color('black');1]],with_box=%f,opt="lr");
      legends(['e1 for L1 = '+string(L1)+' m','e2 for L2 = '+string(L2)+' m','e2-e1','e1/e2'],[color('blue'),color('red'),color('green'),color('black')],with_box=%f,opt="ul"); // upper/lower; left/right
      xtitle("Q = f(T, e1, e2) "+values+"@"+author,"10 K","rel.");   // Beschriftung: Titel, x-Achse, y-Achse
      filena=filename+"_2_r-b.gif";
      xs2gif(2,filena); // GIF von Fenster #2 machen
      printf('... GIF-Plot of Window %i saved as %s \n\n',2,filena); 
 
      
// Drittes Fenster Plot Temp = f(Blau/Rot)
      
      scf(3);   // Fenster 3
      clf(3);   // lschen
      xselect();    // Fenster auf!
      plot2d(zweier(:,1),zweier(:,2),style=color(col))
      // legends('Ratio (Blue/Red)',color(col),with_box=%f,opt="ul"); // upper/lower; left/right
      xtitle("T = Q(e1/e2)"+values+"@"+author, "Q(e1/e2)","T in Kelvin");
      filena=filename+'_3_quot.gif';
      xs2gif(3,filena); // GIF von Fenster #3 machen
      printf('... GIF-Plot of Window %i saved as %s \n\n',3,filena);

      printf('Temperature function saved under \nwrfile = %s\n',wrfile);
      printf('with matrix-format vierer=[grad,e1,e2,e1zue2]\n\n');
      whos -name vierer

      printf('\n\nfor test: \n-->size(vierer)\n-->vierer \n-->whos -name vierer\n\n');
      printf('Example for reload: \n-->wrfile \n-->clear ''vierer''\n-->load(wrfile) \n-->vierer \n\n');
      printf('End.  \n\n'); 
      wd=getcwd();
      printf('GIF-Files saved at \n%s \n\n-->', wd);
      
abort;


